# await / pyawait

`await` is a tool that waits until TCP endpoints are open.

It is useful for synchronizing the spin-up of interdependent services, such as linked docker containers.

This is yet another alternative to `wait-for-it.sh`.

## Rationale

- extended functionality:
    - ability to wait more than one endpoint at the same time (e.g. `-a host:port -a host:port ...`)
    - ability to set polling interval (e.g. `-i 100ms`)
    - timeout/interval can be provided in different time units (`ns,ms,s,m,h`)
- available as a binary file (size <1mb) without any dependencies
- available via `pip` for convenience

## Installation

Download executable binary:

`sh -c "TARGET=await curl -o $TARGET https://raw.githubusercontent.com/jackcvr/await/main/pyawait/await-$(uname -m) && chmod +x $TARGET"`

or:

`pip install pyawait`

[![PyPI - Version](https://img.shields.io/pypi/v/pyawait.svg)](https://pypi.org/project/pyawait)
[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/pyawait.svg)](https://pypi.org/project/pyawait)

## Usage

```text
Usage: await [-t timeout] [-i interval] [-q] [-v] [-s=false] [-a host:port ...] [command [args]]

  -a value
    	Endpoint to be waiting for, in the form 'host:port'
  -i duration
    	Interval between retries in format N{ns,ms,s,m,h} (default 1s)
  -q	Do not print anything (default false)
  -s	Strict mode: execute command after successful result only (default true)
  -t duration
    	Timeout in format N{ns,ms,s,m,h}, e.g. '5s' == 5 seconds. Zero for no timeout (default 0)
  -v	Verbose mode (default false)
  command args
    	Execute command with arguments after the test finishes (default: if connection succeeded)
```

## Examples

Wait 5 seconds for port 80 on `www.google.com`, and if it is available, echo the message `Google is up`:

```bash
$ await -t 5s -a www.google.com:80 echo "Google is up"
2024/08/23 12:34:32.156464 successfully connected to www.google.com:80
Google is up
```

You can provide few addresses at the same time.

Next command waits 2 seconds for www.google.com:80 and localhost:5000, checking them every 500 milliseconds
with enabled verbose mode and disabled strict mode:

```bash
$ await -t 2s -i 500ms -v -s=false -a www.google.com:80 -a localhost:5000 echo "Printed anyway"
2024/08/23 14:42:03.171442 dial tcp 127.0.0.1:5000: connect: connection refused
2024/08/23 14:42:03.248549 successfully connected to www.google.com:80
2024/08/23 14:42:03.682781 dial tcp 127.0.0.1:5000: connect: connection refused
2024/08/23 14:42:04.183945 dial tcp 127.0.0.1:5000: connect: connection refused
2024/08/23 14:42:04.686111 dial tcp 127.0.0.1:5000: connect: connection refused
2024/08/23 14:42:05.187927 dial tcp: lookup localhost: i/o timeout
timeout error
Printed anyway
```

## License

[MIT](https://spdx.org/licenses/MIT.html) 